'*****************************************************************************************************************************************
' Does Computes by-statistics for all variables in a group and provides them in a table as output in the group's own window
' Usage: group_name.groupstatby(options) classifer_names
'*****************************************************************************************************************************************
' Options:
'   stat= one of the following
'    sum: sums
'    mean: means
'    min: minima
'    max: maxima
'    kur: kurtosis
'    med, median: median
'    skew: skewness
'    na: number of NAs
'    mean: means
'    std: standard deviations
'    count: number of observations 
'    quant={arg} quantile with value given by the argument {arg}, default=.5
' classifier_names: by-group series, or an object (text, svector, string, group) which lists the names of the by-groupseries
'*****************************************************************************************************************************************
' Output: Table view in the group's own window
' Needs: nothing
' Side effects: overwrites and deletes tables named "{%temptab11}" and "_$temptab2"
'*****************************************************************************************************************************************

'Set defaults
%typesallowed="SERIES GROUP SVECTOR"  ' list of types allowed for the calling object
%defaultstat="mean"
%statoption= "stat"
%coroutoption="corout"
%formatoption="format"
%coroutdefaultoption="table"
%coroutlist="table matrix"    
%coroutlistoptions="t table tab table table table mat matrix ma matrix matrix matrix"
!defaultnvarscormatrix=3
!defaultquant=0.5
%statlist="mean std var count sum med max min skew kurt na quant"
%bivarstatlist="cor cov sscp rcor rcov rsscp ucor ucov usscp"
%statlistlabels="Mean Std.Dev. Var. Obs. Sum Median Maximum Minimum Skewness Kurtosis NAs Quantile"
%options="nostd nomean nocount nor noc nom nos l sp"

!transpose=0
%transposeoptions="t trans transpose"
%formatoptions="f form format"
%sampleoptions= "sample smpl sa"

%helpfile=@addinspath+"\groupstatby\groupstatby.txt"
%helpargs="help ?"

%outputoptions=             "rm cm sm   tm     full nol" 
%outputoptionswitches="nor noc nos nom l     sp" 

%temptab1="_$temptab1"
%temptab2="_$temptab2"
%temptab3="_$temptab3"
%tmptxt="_$tmptxt"

!maxeditlength=128
!defaultdigits=6

%digits=@str(!defaultdigits)

%objtype=@upper(@getthistype)
if not(@wfind(%typesallowed, %objtype)) then
	@uiprompt("!Allowed objects: "+%typesallowed+" !")
	stop
endif

%var=@word(%args,1)
if @wfind(%helpargs, @lower(%var)) then
	text {%tmptxt}
	{%tmptxt}.append(file) %helpfile
	_this.display {%tmptxt}.
	delete {%tmptxt}
	stop
endif	

for %topt {%transposeoptions} 
	if @hasoption(%topt) then
		!transpose=1
	endif
next

%oopteval=@winterleave(%outputoptions, %outputoptionswitches)
for %oopt %osw {%outputoptionswitches} 
	%oopt=@upper(%oopt)
	if @hasoption(%oopt) then
		%options=@wdrop(%options, %osw)
	endif
next

%options=@replace(%options,","," ")
%stat=%defaultstat
!quant=0

'Get arguments
%cats=@trim(%args)
if %cats="" and %objtype="POOL" then
endif	

!nseries=1
if @isobject(_this.@name) then
	if %objtype= "GROUP" then 
		%vars=_this.@members
		!nseries=_this.@count
	endif 
	if %objtype= "SERIES" then
		%vars=_this.@name
	endif
	if %objtype= "STRING" then
		%vars=_this.string
		!nseries=@wcount(%vars)
	endif
	if %objtype= "SVECTOR" then
		%vars=@wjoin(_this)
		!nseries=@wcount(%vars)
	endif
endif
if !nseries>1 then
	%statlist=%statlist+" "+%bivarstatlist
	%statlistlabels=%statlistlabels+" Correlations"
endif

%currentsmpl=@pagesmpl
%statsmpl=%currentsmpl
for %sopt {%sampleoptions} 
	if @neqna(@equaloption(%sopt),"") then
		%statsmpl=@upper(@trim(@equaloption(%sopt)))
	endif
next

%ifcond = @stripquotes(@right(%statsmpl, @len(%statsmpl)-@instr(%statsmpl, " IF ")+1))
if not @isempty(%ifcond) then
	%statsmplnoifcond = @left(%statsmpl, @len(%statsmpl)-@instr(%statsmpl, @trim(%ifcond))+1)
else
	%statsmplnoifcond = %statsmpl
endif

'Evaluate option stat
if @neqna(@equaloption(%statoption),"") then
	%stat=@lower(@trim(@equaloption(%statoption)))
	if %stat="median" then
		%stat="med"
	endif
	if %stat="variance" then
		%stat="var"
	endif
	if @instr(%stat, "quant")=1 then
		%stat=@replace(%stat," ","")
		if @instr(%stat, "=") then	
			!quant=@val(@right(%stat, @len(%stat)-@instr(%stat,"=")))
		else
			!quant=!defaultquant
		endif
		%stat="quant"
	endif
	if not @wfind(%statlist,%stat) then
		%stat=%defaultstat
	endif
endif
%stat=@trim(@lower(%stat))

for %format {%formatoptions}
	if @neqna(@equaloption(%format),"") then
		%digits=@trim(@equaloption(%format))
	endif
next

if @wfind(%bivarstatlist,%stat) then 
	if @neqna(@equaloption(%coroutoption),"")  then
		%coroutoption=@lower(@trim(@equaloption(%coroutoption)))
		for %t %t2 {%coroutlistoptions} 
			if @lower(%coroutoption)=%t then
				%coroutoption=%t2
			endif
		next
	else
		if @wcount(%vars)>!defaultnvarscormatrix then
			%coroutoption="matrix"
		else
			%coroutoption="table"
		endif
	endif
endif

'If necessary, run gui to obtain arguments
if @len(%cats)<=0 then
	%qua=@str(!defaultquant)
	!result = @uidialog("caption", "Group by statistics", _
	"edit",%cats,"Enter names of bygroups:", _
	"edit", %statsmpl, "Enter sample", !maxeditlength, _
	"list",%stat,"Enter statistics:",%statlist, _
	"edit",%qua,"Enter quant. value between 0.0 and 1.0:", _
	"Check", !transpose, "Transpose output" , _
	"list",%coroutoption,"Output type of correlations:", %coroutlist, _
	"edit", %digits, "Number of digits:") 
	!quant=@val(%qua)
	if !result = -1 then
		stop
	endif	
endif


'Evaluate argument(s)
if @wcount(%cats)=1 then
	if @instr(%cats, "?") or @instr(%cats, "*") then
		%equations=@wlookup(%cats, "series")
	else
		if @isobject(%cats) then
			if @upper({%cats}.@type)="GROUP" then
					%cats={%cats}.@members
			else
				if @upper({%cats}.@type)="STRING" then
					%cats={%cats}
				else
					if @upper({%cats}.@type)="SVECTOR" then
						%cats=@wjoin({%cats})
					else
						if  @upper({%cats}.@type)="TEXT" then
								%cats=@wjoin({%cats}.@svectornb)
						endif
					endif
				endif
			endif
		endif
	endif
endif


%stt=%stat
if %stat="var" then
	%stt="mean"
endif
for %op %ro mean mean std "std. dev." count obs. med median kurt kurt. max max min min. skew skew. na nas sum sum quant quant.*
	if %stt=%op then
		%searchintab=%ro
		exitloop
	endif
next

%format="f."+%digits

if not @wfind(%options,%statout) then
	if not @wfind(%bivarstatlist, %stat) and @neqna(%stat, "var")  then
		%options=%options+" "+%stat
	else
		%options=%options+" "+%defaultstat
	endif
	if !quant<>0 and @instr(%stat, "quant")=1  then 
		%options=%options+"="+@str(!quant)
	endif
else
	%options=@wdrop(%options, %statout)
endif
%statout="no"+%stat
%options=@replace(%options," ",", ")
smpl {%statsmpl} 
%v1=@word(%vars,1)
freeze(mode=overwrite, {%temptab1}) {%v1}.statby({%options}) {%cats}
' %vars=@wdrop(%vars, %v1)
!row=1 
!col=1
!i=1
for %w {%statlist}
	if %w=%stat then
		%writetext=@word(%statlistlabels, !i)
		if %stat="quant" then
			%writetext=%writetext+"="+@str(!quant)
		endif
		exitloop
	endif
	!i=!i+1
next
%writetext="Descriptive statistics: "+%writetext
{%temptab1}(1,1)=%writetext
while not @isempty({%temptab1}(!row,!col))
	!row=!row+1
wend
!row=!row+1
!startrow=!row
while @neqna(@lower(@trim({%temptab1}(!row,!col))),%searchintab)
	!col=!col+1
wend

if @neqna(%stat, "var") then
	!sourcestartcol=!col
	!dc=!sourcestartcol
else
	!sourcestartcol=2
	!dc=@wcount(%cats)+1
endif
while not @isempty({%temptab1}(!row,!col))
	!row=!row+1
wend
!colwidth=10
!adjust=1.2
if not @wfind(%bivarstatlist, %stat) then
	for %v {%vars}
		if @neqna(%stat,"var") then
			freeze(mode=overwrite, {%temptab2}) {%v}.statby({%options}) {%cats}
			!sourcestartrow=1
			while @neqna(@lower(@trim({%temptab2}(!sourcestartrow, !sourcestartcol))), %searchintab)
				!sourcestartrow=!sourcestartrow+1
			wend
		else
			%ifcond=@stripquotes(@right(%statsmpl, @len(%statsmpl)-@instr(%statsmpl, " IF ")+1))
			smpl @last @last {%ifcond} 
			%scats=@wdelim(%cats," ",",")
			freeze(mode=overwrite, {%temptab2}) @varssby({%v}, {%scats}, %statsmpl) 0
			smpl {%statsmpl}
			!sourcestartrow=3
		endif
		!sourceendrow=!sourcestartrow
		while @neqna({%temptab2}(!sourceendrow, !sourcestartcol),"")
			!sourceendrow=!sourceendrow+1
		wend
		!sourceendrow=!sourceendrow-1
		{%temptab2}.copyrange !sourcestartrow !sourcestartcol !sourceendrow !sourcestartcol {%temptab1} !startrow !dc
		{%temptab1}(!startrow,!dc)=%v
		if @len(%v)*!adjust>!colwidth then
			!colwidth=@len(%v)*!adjust
		endif
		{%temptab1}.setWidth(!dc) !colwidth
		!dc=!dc+1
	next
	!rs=!startrow
else
	!dc=!dc-1
	!startrow=!startrow+1
	%sdum=%statsmpl
	if @instr(%sdum, " IF")>0 then 
		%sdum=%sdum+" AND "
	else
		%sdum=%sdum+" IF "
	endif
	!catcounter=1
	for %cat {%cats}
		%sdum=%sdum+%cat+"=$X"+@str(!catcounter)
		if %cat<>@word(%cats, @wcount(%cats)) then
			%sdum=%sdum+" AND " 
		endif
		{%temptab1}(!startrow-1,!catcounter)=@upper(%cat)
		!catcounter=!catcounter+1
	next
	%crosscats=""
	call crossunique(%cats, %crosscats, " ")
	string test=%coroutoption
	if %coroutoption="table" or @eqna(%coroutoption, "")  then
		for !i=1 to @wcount(%vars)
				%v1=@word(%vars,!i)
				for !j=!i+1 to @wcount(%vars)
					%v2=@word(%vars,!j)
					if @isobject("_$tempgr") then
						delete _$tempgr
					endif
					group _$tempgr {%v1} {%v2}
					!row=!startrow
					%colheader=%v1+","+%v2
					{%temptab1}(!row-1,!dc)=%colheader
					for %crosscat {%crosscats} 
						%crosscat=@stripquotes(%crosscat)
					     %newsample=%sdum
					     !catcounter=1
						for %crossca {%crosscat}
							%replacedummy="$X"+@str(!catcounter)
							%newsample=@replace(%newsample, %replacedummy, @addquotes(@stripquotes(%crossca)))
							if !i=1 and !j=2 then 
								{%temptab1}(!row,!catcounter)=%crossca
							endif
							!catcounter=!catcounter+1
						next
						smpl {%newsample} 
						freeze(mode=overwrite, {%temptab2}) _$tempgr.cor {%stat}
						{%temptab1}(!row,!dc)={%temptab2}(4,2) 
						!row=!row+1
					next
					if @len(%colheader)*!adjust>!colwidth then
						!colwidth=@len(%colheader)*!adjust
					endif
					{%temptab1}.setWidth(!dc) !colwidth
					!dc=!dc+1
				next
		next	
	else
		group _$tempgr {%vars}
		!startrow=!startrow-2
		{%temptab1}.deleterow(!startrow) 10
		!nvar =  _$tempgr.@count 
		!row=!startrow
		for %crosscat {%crosscats} 
		     %newsample=%sdum
		     !catcounter=1
			for %crossca {%crosscat}
				%replacedummy="$X"+@str(!catcounter)
				%newsample=@replace(%newsample, %replacedummy, @addquotes(@stripquotes(%crossca)))
				!catcounter=!catcounter+1
			next
			smpl {%newsample} 
			freeze(mode=overwrite, {%temptab2})  _$tempgr.cor {%stat}
			{%temptab2}(1,1)=@stripquotes(%crosscat)
			{%temptab2}.copytable {%temptab1} !row 1
			!row=!row+!nvar+3
		next
	endif
	delete _$tempgr
	smpl {%statsmpl} 		
	!rs=!startrow-1
endif
!dc=!dc-1
!nclassifiers=@wcount(%cats)
if (not @wfind(%bivarstatlist, %stat)) or @eqna(%coroutoption, "table") then
	if !transpose<=0 then
		{%temptab1}.setlines(!rs,1,!rs,!dc) +b
		{%temptab1}.setlines(!rs,!nclassifiers,!rs,!nclassifiers) +r
		{%temptab1}.setlines(!row,1,!row,!dc) +d
		{%temptab1}.setlines(!rs-1,1,!rs-1,!dc) +d
	else 
		!maxlen=0
		for %v {%vars}
			if @len(%v)*!adjust>!colwidth then
				!colwidth=@len(%v)*!adjust
			endif		
		next
	   	if @isobject(%temptab2) then
				delete {%temptab2}
		endif
		{%temptab1}.setlines(@all) -a -d
		table {%temptab2}
		for !r=!rs to !row-1
			!rtarget=!r-!rs+1
			for !c=1 to !dc
				{%temptab1}.copyrange !r !c !r !c {%temptab2} !c !rtarget
			next
		next
		!nr={%temptab2}.@rows
		!nc={%temptab2}.@cols
           {%temptab2}.setlines(@all) -a -d
           {%temptab2}.setlines(1,1,!nr,1) +r 
           {%temptab2}.setlines(!nclassifiers,1,!nclassifiers,!nc) +b 
           {%temptab2}.setlines(!nr+1,1,!nr+1,!nc) +d
		{%temptab2}.setWidth(1) !colwidth
		{%temptab2}.setjust(2,1,!nr,1) left
		{%temptab2}.setjust(2,2,!dc,!nr) middle
           {%temptab2}.insertrow(1) 1
           {%temptab2}.setlines(1,1,1,!nc) +d
		!rstarget=!rs-1
           {%temptab1}.deleterow(!rstarget) !r	
		!copyrows={%temptab1}.@rows
           {%temptab2}.insertrow(1) !copyrows
		{%temptab1}.copytable {%temptab2} 1 1
		{%temptab1}={%temptab2}
	endif
else
	{%temptab1}.insertrow(!startrow) 1
endif
string test=%digits
{%temptab1}.setformat(@all) {%format}
_this.display {%temptab1}
delete {%temptab1} {%temptab2}
smpl {%currentsmpl}


subroutine crossunique(string %cats, string %outstr, string %delim)
	if @isobject(%cats) then
		if {%cats}.@type="GROUP" then
			%cats={%cats}.@members
		endif
		if {%cats}.@type="SVECTOR" then
			%cats=@wjoin({%cats})
		endif
		if {%cats}.@type="SERIES" then
			%cats={%cats}.@name
		endif	
	endif
	%outstr=""
	%pattern="?"+%delim+"?"
	!_$k=1
	for %cat {%cats}
		if @isobject(%cat) then
			if {%cat}.@type="ALPHA" or {%cat}.@detailedtype="SVECTOR" then
				%catstr=@wjoin(@uniquevals({%cat}))
			endif
			if {%cat}.@type="SERIES" or {%cat}.@detailedtype="VECTOR" then
				%catstr=""
				vector _$tempvec=@uniquevals({%cat})
				for !_$i=1 to _$tempvec.@rows
					%catstr=%catstr+@str(_$tempvec(!_$i))+" "
				next
				%catstr=@trim(%catstr)
			endif	
		endif
		if @isempty(%outstr) then
			%outstr=%catstr
		else
			%outstr=@wcross(%outstr, %catstr, %pattern)
		endif
	next
endsub


